<?php
// $Id: FeedsSelfNodeProcessor.inc,v 1.1.2.1 2011/01/27 08:54:39 twistor Exp $

/**
 * @file
 * Class definition of FeedsSelfNodeProcessor.
 */

/**
 * Allows a feed node to populate fields on itself.
 */
class FeedsSelfNodeProcessor extends FeedsNodeProcessor {

  /**
   * Override parent:entityLoad(). Change the log note.
   */
  protected function entityLoad(FeedsSource $source, $nid) {
    $node = parent::entityLoad($source, $nid);
    $node->log = 'Updated by FeedsSelfNodeProcessor';
    return $node;
  }

  /**
   * Process the result of the parsing stage.
   *
   * @param FeedsSource $source
   *   Source information about this import.
   * @param FeedsParserResult $parser_result
   *   The result of the parsing stage.
   */
  public function process(FeedsSource $source, FeedsParserResult $parser_result) {
    $item = reset($parser_result->items);
    $parser_result->current_item = $item;

      // Only proceed if item has actually changed.
    $hash = $this->hash($item);
    $entity_id = $source->feed_nid;

    if ($hash == $this->getHash($entity_id)) {
      return;
    }

    try {
      // Assemble node, map item to it, save.
      $entity = $this->entityLoad($source, $entity_id);
      // If an existing item info can't be loaded, create one.
      if (!$this->loadItemInfo($entity)) {
        $this->newItemInfo($entity, $source->feed_nid, $hash);
        $entity->feeds_item->entity_id = $entity_id;
      }
      $this->map($source, $parser_result, $entity);
      $this->entityValidate($entity);
      $this->entitySave($entity);
    }
    catch (Exception $e) {
      drupal_set_message($e->getMessage(), 'warning');
      $message = $e->getMessage();
      $message .= '<h3>Original item</h3>';
      $message .= '<pre>' . var_export($item, true) . '</pre>';
      $message .= '<h3>Entity</h3>';
      $message .= '<pre>' . var_export($entity, true) . '</pre>';
      $source->log('import', $message, array(), WATCHDOG_ERROR);
      return;
    }
    $message = t('@type %title has been updated.', array('@type' => node_type_get_name($entity), '%title' => $entity->title));
    drupal_set_message($message);
    $source->log('import', $message, array(), WATCHDOG_INFO);
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    $defaults = parent::configDefaults();
    $defaults['update_existing'] = FEEDS_UPDATE_EXISTING;
    $defaults['expire'] = FEEDS_EXPIRE_NEVER;
    return $defaults;
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = parent::configForm($form_state);
    $form['expire']['#type'] = 'hidden';
    unset($form['update_existing']);
    unset($form['content_type']);
    $form['author']['#type'] = 'hidden';
    return $form;
  }
}
